/*

    Main_menu class code

*/

#include "SMenu.hpp"
#include <list>

void		UpdateVertex(IrisManualObject &image, int index, vect3d &motion)
{
    vect3d	&pos = image.GetVertex(index);

    // move the vertex with the motion vector
    pos += motion;
    // color the vertex
    /*
    float	red = pos._x / 150.0f;
    red = (red < 0.0f) ? 0.0f : (red > 1.0f) ? 1.0f : red;
    float	green = (pos._y - 360.0f) / 120.0f;
    green = (green < 0.0f) ? 0.0f : (green > 1.0f) ? 1.0f : green;
    image.SetColor(index, IrisColor(red, green, 1.0f, 1.0f));
    */

    // modify the motion vector if vertex is out of the screen
    if (pos._x <= 0.0f || 340.0f <= pos._x)
        motion._x *= -1.0f;
    if (pos._y <= 240.0f || 480.0f <=pos._y)
        motion._y *= -1.0f;
}

SMenu::SMenu(char *named, char *background):ctx(IrisContext::Get())
{
    name = named;
    back_name = background;
}

/*
    Loop of the main menu, displays the menu and checks controllers to enter
    the other menus.
*/
int SMenu::run()
{
    // Manual object declaration for the menu background
    printf("== SMenu::run() == Declarations\n");
    IrisManualObject	back;
    IrisMaterial		*material = 0;
    vect3d				tr_up(0.0f, -1.0f, 0.0f);
    vect3d				tr_down(0.0f, 1.0f, 0.0f);
    vect3d				tr(0.0f, 0.0f, 0.0f);
    bool				in_menu = true;
    int					choice = 0;
    int					num_opt = 2;

    // Code from Iris tutorial !!
    // ****

    // setup project, viewport and camera(view) matrix
    ctx.GetGeometryPipeline().SetProjection(90.0f, 1.0f, 1000.0f);
    ctx.GetGeometryPipeline().SetViewport(0.0f, 0.0f, 640.0f, 480.0f);
    ctx.GetGeometryPipeline().SetView(vect3d(0.0f, 100.0f, 0.0f),
                                    vect3d(0.0f, 0.0f, 1.0f),
                                    vect3d(0.0f, 1.0f, 0.0f));
    ctx.GetGeometryPipeline().Update();
    printf("== SMenu::run() == Geometry pipeline complete\n");
    // set ambient color
    ctx.GetLightingPipeline().SetAmbientColor(IrisColor(1.0f, 1.0f, 1.0f));
    ctx.GetLightingPipeline().Update();
    printf("== SMenu::run() == Lightning pipeline complete\n");
    // get the first paddle in mapple bus
    IrisController &ctrl = ctx.GetInputManager().GetController(0);
    printf("== SMenu::run() == Controler get complete\n");

    // ****
    material = ctx.GetMaterialManager().LoadMaterial(this->back_name);
    printf("== SMenu::run() == Material loading complete\n");
    back.Allocate(4);
    back.GetVertex(0) = vect3d(0.0f, 25.0f, 1.0f);
    //385;75
    back.GetVertex(1) = vect3d(640.0f, 25.0f, 1.0f);
    back.GetVertex(2) = vect3d(0.0f, 455.0f, 1.0f);
    back.GetVertex(3) = vect3d(640.0f, 455.0f, 1.0f);
    printf("== SMenu::run() == Vextexes allocating and setting complete\n");
    back.SetColor(IrisColor(1.0f, 1.0f, 1.0f, 1.0f)); // RGB white ?
    // Apply the texture
    back.SetMaterial(material);
    // UV coordinates to apply the texture
    back.SetUV(0, 0.0f, 0.0f);
    back.SetUV(1, 1.0f, 0.0f);
    back.SetUV(2, 0.0f, 1.0f);
    back.SetUV(3, 1.0f, 1.0f);
    // Prerendering (3D -> 2D)
    back.Precalculate2D();

    // Loop to display the menu
    while (in_menu)
    {
        if (ctrl.Up())
        {
            printf("D-pad Up pressed\n");
            if (--choice < 0)
            	choice = num_opt;
           	//tr = tr_up;
           	material = ctx.GetMaterialManager().LoadMaterial(this->back_name);
           	back.SetMaterial(material);
        }
       	if (ctrl.Down())
       	{
       	    printf("D-pad Down pressed\n");
            if (++choice >= num_opt)
            	choice = 0;
            //tr = tr_down;
            material = ctx.GetMaterialManager().LoadMaterial("/rd/menustart2.png");
            back.SetMaterial(material);
        }
        if (ctrl.y())
        	break;
        if (ctrl.a() || ctrl.Start())
        {
            if (choice == 0)
            {
           		SMenu	hop("hop", "/rd/menu2.png");
           		hop.run();
      		}
           	printf("A or Start pressed\n");
        }
       	/*
       	UpdateVertex(back, 0, tr);
       	UpdateVertex(back, 1, tr);
       	UpdateVertex(back, 2, tr);
       	UpdateVertex(back, 3, tr);
       	*/
       	back.Precalculate2D();
        ctx.BeginScene();
        ctx.SetDepthWrite(false);
        back.Render2D();
        ctx.SetDepthWrite(true);
        ctx.EndScene();
    }
    return 0;
}
